@@ -13,7 +13,7 @@
 ** is not included in the SQLite library.  It is used for automated
 ** testing of the SQLite library.
 **
-** $Id: test8.c,v 1.57 2007/09/03 15:03:21 danielk1977 Exp $
+** $Id: test8.c,v 1.74 2008/08/20 14:49:25 danielk1977 Exp $
 */
 #include "sqliteInt.h"
 #include "tcl.h"
@@ -44,6 +44,22 @@ typedef struct echo_cursor echo_cursor;
 ** use the named table as a backing store will fail.
 */
 
+/*
+** Errors can be provoked within the following echo virtual table methods:
+**
+**   xBestIndex   xOpen     xFilter   xNext   
+**   xColumn      xRowid    xUpdate   xSync   
+**   xBegin       xRename
+**
+** This is done by setting the global tcl variable:
+**
+**   echo_module_fail($method,$tbl)
+**
+** where $method is set to the name of the virtual table method to fail
+** (i.e. "xBestIndex") and $tbl is the name of the table being echoed (not
+** the name of the virtual table, the name of the underlying real table).
+*/
+
 /* 
 ** An echo virtual-table object.
 **
@@ -61,6 +77,7 @@ struct echo_vtab {
   sqlite3 *db;            /* Database connection */
 
   int isPattern;
+  int inTransaction;      /* True if within a transaction */
   char *zThis;            /* Name of the echo table */
   char *zTableName;       /* Name of the real table */
   char *zLogName;         /* Name of the log table */
@@ -75,6 +92,18 @@ struct echo_cursor {
   sqlite3_stmt *pStmt;
 };
 
+static int simulateVtabError(echo_vtab *p, const char *zMethod){
+  const char *zErr;
+  char zVarname[128];
+  zVarname[127] = '\0';
+  snprintf(zVarname, 127, "echo_module_fail(%s,%s)", zMethod, p->zTableName);
+  zErr = Tcl_GetVar(p->interp, zVarname, TCL_GLOBAL_ONLY);
+  if( zErr ){
+    p->base.zErrMsg = sqlite3_mprintf("echo-vtab-error: %s", zErr);
+  }
+  return (zErr!=0);
+}
+
 /*
 ** Convert an SQL-style quoted string into a normal string by removing
 ** the quote characters.  The conversion is done in-place.  If the
@@ -160,7 +189,12 @@ static int getColumnNames(
     */
     nBytes = sizeof(char *) * nCol;
     for(ii=0; ii<nCol; ii++){
-      nBytes += (strlen(sqlite3_column_name(pStmt, ii)) + 1);
+      const char *zName = sqlite3_column_name(pStmt, ii);
+      if( !zName ){
+        rc = SQLITE_NOMEM;
+        goto out;
+      }
+      nBytes += strlen(zName)+1;
     }
     aCol = (char **)sqlite3MallocZero(nBytes);
     if( !aCol ){
@@ -453,7 +487,7 @@ static int echoCreate(
     rc = sqlite3_exec(db, zSql, 0, 0, 0);
     sqlite3_free(zSql);
     if( rc!=SQLITE_OK ){
-      *pzErr = sqlite3StrDup(sqlite3_errmsg(db));
+      *pzErr = sqlite3DbStrDup(0, sqlite3_errmsg(db));
     }
   }
 
@@ -462,6 +496,10 @@ static int echoCreate(
     *ppVtab = 0;
   }
 
+  if( rc==SQLITE_OK ){
+    (*(echo_vtab**)ppVtab)->inTransaction = 1;
+  }
+
   return rc;
 }
 
@@ -514,6 +552,9 @@ static int echoDestroy(sqlite3_vtab *pVtab){
 */
 static int echoOpen(sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor){
   echo_cursor *pCur;
+  if( simulateVtabError((echo_vtab *)pVTab, "xOpen") ){
+    return SQLITE_ERROR;
+  }
   pCur = sqlite3MallocZero(sizeof(echo_cursor));
   *ppCursor = (sqlite3_vtab_cursor *)pCur;
   return (pCur ? SQLITE_OK : SQLITE_NOMEM);
@@ -547,6 +588,10 @@ static int echoNext(sqlite3_vtab_cursor *cur){
   int rc = SQLITE_OK;
   echo_cursor *pCur = (echo_cursor *)cur;
 
+  if( simulateVtabError((echo_vtab *)(cur->pVtab), "xNext") ){
+    return SQLITE_ERROR;
+  }
+
   if( pCur->pStmt ){
     rc = sqlite3_step(pCur->pStmt);
     if( rc==SQLITE_ROW ){
@@ -566,6 +611,11 @@ static int echoNext(sqlite3_vtab_cursor *cur){
 static int echoColumn(sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i){
   int iCol = i + 1;
   sqlite3_stmt *pStmt = ((echo_cursor *)cur)->pStmt;
+
+  if( simulateVtabError((echo_vtab *)(cur->pVtab), "xColumn") ){
+    return SQLITE_ERROR;
+  }
+
   if( !pStmt ){
     sqlite3_result_null(ctx);
   }else{
@@ -580,6 +630,11 @@ static int echoColumn(sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i){
 */
 static int echoRowid(sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid){
   sqlite3_stmt *pStmt = ((echo_cursor *)cur)->pStmt;
+
+  if( simulateVtabError((echo_vtab *)(cur->pVtab), "xRowid") ){
+    return SQLITE_ERROR;
+  }
+
   *pRowid = sqlite3_column_int64(pStmt, 0);
   return SQLITE_OK;
 }
@@ -617,6 +672,10 @@ static int echoFilter(
   echo_vtab *pVtab = (echo_vtab *)pVtabCursor->pVtab;
   sqlite3 *db = pVtab->db;
 
+  if( simulateVtabError(pVtab, "xFilter") ){
+    return SQLITE_ERROR;
+  }
+
   /* Check that idxNum matches idxStr */
   assert( idxNum==hashString(idxStr) );
 
@@ -697,7 +756,7 @@ static void string_concat(char **pzStr, char *zAppend, int doFree, int *pRc){
 **
 ** then the echo module handles WHERE or ORDER BY clauses that refer
 ** to the column "b", but not "a" or "c". If a multi-column index is
-** present, only it's left most column is considered. 
+** present, only its left most column is considered. 
 **
 ** This xBestIndex method encodes the proposed search strategy as
 ** an SQL query on the real table underlying the virtual echo module 
@@ -724,6 +783,14 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
   int rc = SQLITE_OK;
   int useCost = 0;
   double cost;
+  int isIgnoreUsable = 0;
+  if( Tcl_GetVar(interp, "echo_module_ignore_usable", TCL_GLOBAL_ONLY) ){
+    isIgnoreUsable = 1;
+  }
+
+  if( simulateVtabError(pVtab, "xBestIndex") ){
+    return SQLITE_ERROR;
+  }
 
   /* Determine the number of rows in the table and store this value in local
   ** variable nRow. The 'estimated-cost' of the scan will be the number of
@@ -763,8 +830,10 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
     pConstraint = &pIdxInfo->aConstraint[ii];
     pUsage = &pIdxInfo->aConstraintUsage[ii];
 
+    if( !isIgnoreUsable && !pConstraint->usable ) continue;
+
     iCol = pConstraint->iColumn;
-    if( pVtab->aIndex[iCol] ){
+    if( pVtab->aIndex[iCol] || iCol<0 ){
       char *zCol = pVtab->aCol[iCol];
       char *zOp = 0;
       useIdx = 1;
@@ -871,6 +940,14 @@ int echoUpdate(
 
   assert( nData==pVtab->nCol+2 || nData==1 );
 
+  /* Ticket #3083 - make sure we always start a transaction prior to
+  ** making any changes to a virtual table */
+  assert( pVtab->inTransaction );
+
+  if( simulateVtabError(pVtab, "xUpdate") ){
+    return SQLITE_ERROR;
+  }
+
   /* If apData[0] is an integer and nData>1 then do an UPDATE */
   if( nData>1 && sqlite3_value_type(apData[0])==SQLITE_INTEGER ){
     char *zSep = " SET";
@@ -952,16 +1029,23 @@ int echoUpdate(
     if( bindArgOne ){
       sqlite3_bind_value(pStmt, 1, apData[1]);
     }
-    for(i=2; i<nData; i++){
-      if( apData[i] ) sqlite3_bind_value(pStmt, i, apData[i]);
+    for(i=2; i<nData && rc==SQLITE_OK; i++){
+      if( apData[i] ) rc = sqlite3_bind_value(pStmt, i, apData[i]);
+    }
+    if( rc==SQLITE_OK ){
+      sqlite3_step(pStmt);
+      rc = sqlite3_finalize(pStmt);
+    }else{
+      sqlite3_finalize(pStmt);
     }
-    sqlite3_step(pStmt);
-    rc = sqlite3_finalize(pStmt);
   }
 
   if( pRowid && rc==SQLITE_OK ){
     *pRowid = sqlite3_last_insert_rowid(db);
   }
+  if( rc!=SQLITE_OK ){
+    tab->zErrMsg = sqlite3_mprintf("echo-vtab-error: %s", sqlite3_errmsg(db));
+  }
 
   return rc;
 }
@@ -975,10 +1059,11 @@ static int echoTransactionCall(sqlite3_vtab *tab, const char *zCall){
   char *z;
   echo_vtab *pVtab = (echo_vtab *)tab;
   z = sqlite3_mprintf("echo(%s)", pVtab->zTableName);
+  if( z==0 ) return SQLITE_NOMEM;
   appendToEchoModule(pVtab->interp, zCall);
   appendToEchoModule(pVtab->interp, z);
   sqlite3_free(z);
-  return (z?SQLITE_OK:SQLITE_NOMEM);
+  return SQLITE_OK;
 }
 static int echoBegin(sqlite3_vtab *tab){
   int rc;
@@ -986,6 +1071,14 @@ static int echoBegin(sqlite3_vtab *tab){
   Tcl_Interp *interp = pVtab->interp;
   const char *zVal; 
 
+  /* Ticket #3083 - do not start a transaction if we are already in
+  ** a transaction */
+  assert( !pVtab->inTransaction );
+
+  if( simulateVtabError(pVtab, "xBegin") ){
+    return SQLITE_ERROR;
+  }
+
   rc = echoTransactionCall(tab, "xBegin");
 
   if( rc==SQLITE_OK ){
@@ -998,6 +1091,9 @@ static int echoBegin(sqlite3_vtab *tab){
       rc = SQLITE_ERROR;
     }
   }
+  if( rc==SQLITE_OK ){
+    pVtab->inTransaction = 1;
+  }
   return rc;
 }
 static int echoSync(sqlite3_vtab *tab){
@@ -1006,6 +1102,14 @@ static int echoSync(sqlite3_vtab *tab){
   Tcl_Interp *interp = pVtab->interp;
   const char *zVal; 
 
+  /* Ticket #3083 - Only call xSync if we have previously started a
+  ** transaction */
+  assert( pVtab->inTransaction );
+
+  if( simulateVtabError(pVtab, "xSync") ){
+    return SQLITE_ERROR;
+  }
+
   rc = echoTransactionCall(tab, "xSync");
 
   if( rc==SQLITE_OK ){
@@ -1021,11 +1125,34 @@ static int echoSync(sqlite3_vtab *tab){
   return rc;
 }
 static int echoCommit(sqlite3_vtab *tab){
-  sqlite3MallocBenignFailure(1);
-  return echoTransactionCall(tab, "xCommit");
+  echo_vtab *pVtab = (echo_vtab*)tab;
+  int rc;
+
+  /* Ticket #3083 - Only call xCommit if we have previously started
+  ** a transaction */
+  assert( pVtab->inTransaction );
+
+  if( simulateVtabError(pVtab, "xCommit") ){
+    return SQLITE_ERROR;
+  }
+
+  sqlite3BeginBenignMalloc();
+  rc = echoTransactionCall(tab, "xCommit");
+  sqlite3EndBenignMalloc();
+  pVtab->inTransaction = 0;
+  return rc;
 }
 static int echoRollback(sqlite3_vtab *tab){
-  return echoTransactionCall(tab, "xRollback");
+  int rc;
+  echo_vtab *pVtab = (echo_vtab*)tab;
+
+  /* Ticket #3083 - Only call xRollback if we have previously started
+  ** a transaction */
+  assert( pVtab->inTransaction );
+
+  rc = echoTransactionCall(tab, "xRollback");
+  pVtab->inTransaction = 0;
+  return rc;
 }
 
 /*
@@ -1091,6 +1218,10 @@ static int echoRename(sqlite3_vtab *vtab, const char *zNewName){
   int rc = SQLITE_OK;
   echo_vtab *p = (echo_vtab *)vtab;
 
+  if( simulateVtabError(p, "xRename") ){
+    return SQLITE_ERROR;
+  }
+
   if( p->isPattern ){
     int nThis = strlen(p->zThis);
     char *zSql = sqlite3MPrintf(0, "ALTER TABLE %s RENAME TO %s%s", 
@@ -1133,10 +1264,7 @@ static sqlite3_module echoModule = {
 /*
 ** Decode a pointer to an sqlite3 object.
 */
-static int getDbPointer(Tcl_Interp *interp, const char *zA, sqlite3 **ppDb){
-  *ppDb = (sqlite3*)sqlite3TextToPtr(zA);
-  return TCL_OK;
-}
+extern int getDbPointer(Tcl_Interp *interp, const char *zA, sqlite3 **ppDb);
 
 static void moduleDestroy(void *p){
   sqlite3_free(p);
@@ -1196,20 +1324,20 @@ static int declare_vtab(
 ** Register commands with the TCL interpreter.
 */
 int Sqlitetest8_Init(Tcl_Interp *interp){
+#ifndef SQLITE_OMIT_VIRTUALTABLE
   static struct {
      char *zName;
      Tcl_ObjCmdProc *xProc;
      void *clientData;
   } aObjCmd[] = {
-#ifndef SQLITE_OMIT_VIRTUALTABLE
      { "register_echo_module",   register_echo_module, 0 },
      { "sqlite3_declare_vtab",   declare_vtab, 0 },
-#endif
   };
   int i;
   for(i=0; i<sizeof(aObjCmd)/sizeof(aObjCmd[0]); i++){
     Tcl_CreateObjCommand(interp, aObjCmd[i].zName, 
         aObjCmd[i].xProc, aObjCmd[i].clientData, 0);
   }
+#endif
   return TCL_OK;
 }
