@@ -13,6 +13,8 @@
 ** This file contains code that modified the OS layer in order to simulate
 ** different device types (by overriding the return values of the 
 ** xDeviceCharacteristics() and xSectorSize() methods).
+**
+** $Id: test_devsym.c,v 1.7 2008/06/06 11:11:26 danielk1977 Exp $
 */
 #if SQLITE_TEST          /* This file is used for testing only */
 
@@ -50,10 +52,6 @@ static int devsymCheckReservedLock(sqlite3_file*, int *);
 static int devsymFileControl(sqlite3_file*, int op, void *pArg);
 static int devsymSectorSize(sqlite3_file*);
 static int devsymDeviceCharacteristics(sqlite3_file*);
-static int devsymShmLock(sqlite3_file*,int,int,int);
-static int devsymShmMap(sqlite3_file*,int,int,int, void volatile **);
-static void devsymShmBarrier(sqlite3_file*);
-static int devsymShmUnmap(sqlite3_file*,int);
 
 /*
 ** Method declarations for devsym_vfs.
@@ -65,7 +63,7 @@ static int devsymFullPathname(sqlite3_vfs*, const char *zName, int, char *zOut);
 #ifndef SQLITE_OMIT_LOAD_EXTENSION
 static void *devsymDlOpen(sqlite3_vfs*, const char *zFilename);
 static void devsymDlError(sqlite3_vfs*, int nByte, char *zErrMsg);
-static void (*devsymDlSym(sqlite3_vfs*,void*, const char *zSymbol))(void);
+static void *devsymDlSym(sqlite3_vfs*,void*, const char *zSymbol);
 static void devsymDlClose(sqlite3_vfs*, void*);
 #endif /* SQLITE_OMIT_LOAD_EXTENSION */
 static int devsymRandomness(sqlite3_vfs*, int nByte, char *zOut);
@@ -73,7 +71,7 @@ static int devsymSleep(sqlite3_vfs*, int microseconds);
 static int devsymCurrentTime(sqlite3_vfs*, double*);
 
 static sqlite3_vfs devsym_vfs = {
-  2,                     /* iVersion */
+  1,                     /* iVersion */
   sizeof(devsym_file),      /* szOsFile */
   DEVSYM_MAX_PATHNAME,      /* mxPathname */
   0,                     /* pNext */
@@ -96,13 +94,11 @@ static sqlite3_vfs devsym_vfs = {
 #endif /* SQLITE_OMIT_LOAD_EXTENSION */
   devsymRandomness,         /* xRandomness */
   devsymSleep,              /* xSleep */
-  devsymCurrentTime,        /* xCurrentTime */
-  0,                        /* xGetLastError */
-  0                         /* xCurrentTimeInt64 */
+  devsymCurrentTime         /* xCurrentTime */
 };
 
 static sqlite3_io_methods devsym_io_methods = {
-  2,                                /* iVersion */
+  1,                            /* iVersion */
   devsymClose,                      /* xClose */
   devsymRead,                       /* xRead */
   devsymWrite,                      /* xWrite */
@@ -114,11 +110,7 @@ static sqlite3_io_methods devsym_io_methods = {
   devsymCheckReservedLock,          /* xCheckReservedLock */
   devsymFileControl,                /* xFileControl */
   devsymSectorSize,                 /* xSectorSize */
-  devsymDeviceCharacteristics,      /* xDeviceCharacteristics */
-  devsymShmMap,                     /* xShmMap */
-  devsymShmLock,                    /* xShmLock */
-  devsymShmBarrier,                 /* xShmBarrier */
-  devsymShmUnmap                    /* xShmUnmap */
+  devsymDeviceCharacteristics       /* xDeviceCharacteristics */
 };
 
 struct DevsymGlobal {
@@ -232,34 +224,6 @@ static int devsymDeviceCharacteristics(sqlite3_file *pFile){
   return g.iDeviceChar;
 }
 
-/*
-** Shared-memory methods are all pass-thrus.
-*/
-static int devsymShmLock(sqlite3_file *pFile, int ofst, int n, int flags){
-  devsym_file *p = (devsym_file *)pFile;
-  return sqlite3OsShmLock(p->pReal, ofst, n, flags);
-}
-static int devsymShmMap(
-  sqlite3_file *pFile, 
-  int iRegion, 
-  int szRegion, 
-  int isWrite, 
-  void volatile **pp
-){
-  devsym_file *p = (devsym_file *)pFile;
-  return sqlite3OsShmMap(p->pReal, iRegion, szRegion, isWrite, pp);
-}
-static void devsymShmBarrier(sqlite3_file *pFile){
-  devsym_file *p = (devsym_file *)pFile;
-  sqlite3OsShmBarrier(p->pReal);
-}
-static int devsymShmUnmap(sqlite3_file *pFile, int delFlag){
-  devsym_file *p = (devsym_file *)pFile;
-  return sqlite3OsShmUnmap(p->pReal, delFlag);
-}
-
-
-
 /*
 ** Open an devsym file handle.
 */
@@ -270,14 +234,10 @@ static int devsymOpen(
   int flags,
   int *pOutFlags
 ){
-  int rc;
   devsym_file *p = (devsym_file *)pFile;
+  pFile->pMethods = &devsym_io_methods;
   p->pReal = (sqlite3_file *)&p[1];
-  rc = sqlite3OsOpen(g.pVfs, zName, p->pReal, flags, pOutFlags);
-  if( p->pReal->pMethods ){
-    pFile->pMethods = &devsym_io_methods;
-  }
-  return rc;
+  return sqlite3OsOpen(g.pVfs, zName, p->pReal, flags, pOutFlags);
 }
 
 /*
@@ -336,8 +296,8 @@ static void devsymDlError(sqlite3_vfs *pVfs, int nByte, char *zErrMsg){
 /*
 ** Return a pointer to the symbol zSymbol in the dynamic library pHandle.
 */
-static void (*devsymDlSym(sqlite3_vfs *pVfs, void *p, const char *zSym))(void){
-  return sqlite3OsDlSym(g.pVfs, p, zSym);
+static void *devsymDlSym(sqlite3_vfs *pVfs, void *pHandle, const char *zSymbol){
+  return sqlite3OsDlSym(g.pVfs, pHandle, zSymbol);
 }
 
 /*
@@ -368,10 +328,9 @@ static int devsymSleep(sqlite3_vfs *pVfs, int nMicro){
 ** Return the current time as a Julian Day number in *pTimeOut.
 */
 static int devsymCurrentTime(sqlite3_vfs *pVfs, double *pTimeOut){
-  return g.pVfs->xCurrentTime(g.pVfs, pTimeOut);
+  return sqlite3OsCurrentTime(g.pVfs, pTimeOut);
 }
 
-
 /*
 ** This procedure registers the devsym vfs with SQLite. If the argument is
 ** true, the devsym vfs becomes the new default vfs. It is the only publicly
@@ -385,13 +344,9 @@ void devsym_register(int iDeviceChar, int iSectorSize){
   }
   if( iDeviceChar>=0 ){
     g.iDeviceChar = iDeviceChar;
-  }else{
-    g.iDeviceChar = 0;
   }
   if( iSectorSize>=0 ){
     g.iSectorSize = iSectorSize;
-  }else{
-    g.iSectorSize = 512;
   }
 }
 
